/*
 * MultiMail offline mail reader
 * some low-level routines common to both sides

 Copyright (c) 2000 William McBrine <wmcbrine@clark.net>

 Distributed under the GNU General Public License.
 For details, see the file COPYING in the parent directory. */

/* Most non-ANSI, non-curses stuff is here. */

#include "interfac.h"
#include "error.h"

extern "C" {
#include <dirent.h>

#ifdef USE_SPAWNO
# include <spawno.h>
#endif

#ifdef LIMIT_MEM
# include <alloc.h>
#endif

#ifdef HAS_UNISTD
# include <unistd.h>
#else
# include <dir.h>
#endif

#include <sys/stat.h>

#ifdef HAS_UNAME
# include <sys/utsname.h>
#endif

#ifdef __EMX__
int _chdir2(const char *);
char *_getcwd2(char *, int);
#endif
}

static DIR *Dir;
static dirent *entry;

void fatalError(const char *description);

char *myfgets(char *s, size_t size, FILE *stream)
{
	char *end = 0;

	if (!feof(stream) && fgets(s, size, stream)) {

		end = s + strlen(s) - 1;

		// Skip any leftovers:
		if (*end != '\n')
			while (!feof(stream) && (fgetc(stream) != '\n'));

	}
	return end;
}

int mysystem(const char *cmd)
{
	if (ui) {
		if (!isendwin())
			endwin();
#if defined (__PDCURSES__) && !defined (XCURSES)
		// Restore original cursor
		PDC_set_cursor_mode(curs_start, curs_end);
#endif
	}

#ifdef USE_SPAWNO
	const char *tmp = getenv("TEMP");

	if (!tmp) {
		tmp = getenv("TMP");
		if (!tmp)
			tmp = error.getOrigDir();
	}

	int result = mm.resourceObject->getInt(swapOut) ?
		systemo(tmp, cmd) : -1;

	if (-1 == result)
		result = system(cmd);
#else
	int result = system(cmd);
#endif

#ifdef HAS_SLEEP
	// Non-zero result = error; pause so it can (maybe) be read
	if (result)
		sleep(2);
#endif

	if (ui) {
		keypad(stdscr, TRUE);
#ifdef NOTYPEAHEAD
		typeahead(-1);
#endif
	}

	return result;
}

void mytmpnam(char *name)
{
/* EMX doesn't return an absolute pathname from tmpnam(), so we create one
   ourselves. Otherwise, use the system's version, and make sure it hasn't
   run out of names.
*/
#ifdef TEMP_RELATIVE
	const char *tmp = getenv("TEMP");

	if (!tmp) {
		tmp = getenv("TMP");
		if (!tmp)
			tmp = error.getOrigDir();
	}
	char end = tmp[strlen(tmp) - 1];
	sprintf(name, (('/' == end) || ('\\' == end)) ? "%s%s" : "%s/%s",
		tmp, tmpnam(0));
#else
	if (!tmpnam(name))
		fatalError("Out of temporary filenames");
#endif
}

void edit(const char *reply_filename)
{
        char command[512];

        sprintf(command, "%.255s %.255s", mm.resourceObject->get(editor),
                canonize(reply_filename));
        mysystem(command);
}

int mychdir(const char *pathname)
{
#ifdef USE_SETDISK
	if (':' == pathname[1])
		setdisk(toupper(pathname[0]) - 'A');
#endif
	return
#ifdef __EMX__
		_chdir2(pathname);
#else
		chdir(pathname);
#endif
}

int mymkdir(const char *pathname)
{
#ifdef HAS_UNISTD
	return mkdir(pathname, S_IRWXU);
#else
	return mkdir(pathname);
#endif
}

void myrmdir(const char *pathname)
{
	rmdir(pathname);
}

void mygetcwd(char *pathname)
{
#ifdef __EMX__
	_getcwd2(pathname, 255);
#else
	getcwd(pathname, 255);
#endif
}

bool readable(const char *filename)
{
#ifdef HAS_UNISTD
	return !access(filename, R_OK);
#else
	mystat foo;
	bool exists = foo.init(filename);
	return exists;
#endif
}

bool writeable(const char *filename)
{
#ifdef HAS_UNISTD
	return !access(filename, R_OK | W_OK);
#else
	// This could surely be improved upon
	return readable(filename);
#endif
}

// system name -- results of uname()
const char *sysname()
{
#ifdef HAS_UNAME
	static struct utsname buf;

	if (!buf.sysname[0])
		uname(&buf);

	return buf.sysname;
#else
# ifdef __WIN32__
	return "Win32";
# else
	return "XT";
# endif
#endif
}

bool myopendir(const char *dirname)
{
	return ((Dir = opendir((char *) dirname)) != 0) ?
		!mychdir(dirname) : false;
}

const char *myreaddir()
{
	entry = readdir(Dir);
	if (!entry) {
		closedir(Dir);
		return 0;
	} else
		return entry->d_name;
}

void clearDirectory(const char *DirName)
{
	const char *fname;

	if (myopendir(DirName))
		while ((fname = myreaddir()) != 0)
			if (fname[0] != '.')
				remove(fname);
}

#ifdef LIMIT_MEM

/* Constrain memory allocation according to maximum block size and free
   memory remaining. Currently used only in the 16-bit MS-DOS port.
*/

long limitmem(long wanted)
{
	long maxavail = (long) coreleft();

	// Give it a 25% margin
	maxavail -= (wanted >> 2);

	if (maxavail > MAXBLOCK)
		maxavail = MAXBLOCK;

	if (wanted > maxavail)
		wanted = maxavail;

	return wanted;
}

#endif

#ifdef DOSNAMES

/* Convert pathnames to "canonical" form (change slashes to backslashes).
   The "nospace" stuff leaves any parameters unconverted.
   Don't call this twice in a row without first copying the result! D'oh!
*/

const char *canonize(const char *sinner)
{
	static char saint[256];
	int i;
	bool nospace = true;

	for (i = 0; sinner[i]; i++) {
		saint[i] = (nospace && (sinner[i] == '/')) ?
			'\\' : sinner[i];
		if (nospace && (saint[i] == ' '))
			nospace = false;
	}
	saint[i] = '\0';
	return saint;
}

#endif

#ifdef USE_SHELL

/* Command shell routine -- currently only used in the DOSish ports */

Shell::Shell()
{
	const char *oldprompt = getenv("PROMPT");
	if (!oldprompt)
		oldprompt = "$p$g";

	int len = strlen(oldprompt) + 13;
	prompt = new char[len];

	sprintf(prompt, "PROMPT=%s[MM] ", oldprompt);
	putenv(prompt);
}

Shell::~Shell()
{
	delete[] prompt;
}

void Shell::out()
{
	mychdir(error.getOrigDir());
	touchwin(stdscr);
	refresh();
	mysystem(getenv("COMSPEC"));

	ui->redraw();
}

#endif

bool mystat::init(const char *fname)
{
	struct stat fileStat;
	bool retval = !stat((char *) fname, &fileStat);
	size = retval ? fileStat.st_size : -1;
	date = retval ? fileStat.st_mtime : (time_t) -1;
#ifdef HAS_UNISTD
	isdir = !(!S_ISDIR(fileStat.st_mode));
#else
	isdir = !(!(fileStat.st_mode & S_IFDIR));
#endif
	return retval;
}
